import { defineStore } from 'pinia';
import { cloneDeep } from 'lodash';
import userApi from '@/api/user';
import router from '@/router/index';
import asyncRoutes from '@/router/asyncRoutes';
import { getTreeList, local } from '@/utils/tool';

// 路由扁平化
const flatAsyncRoutes = (routes, breadcrumb = []) => {
  const res = [];
  routes.forEach((route) => {
    const tmp = { ...route };
    if (tmp.children) {
      const childrenBreadcrumb = [...breadcrumb];
      childrenBreadcrumb.push(route);
      const tmpRoute = { ...route };
      tmpRoute.meta.breadcrumb = childrenBreadcrumb;
      delete tmpRoute.children;
      res.push(tmpRoute);
      const childrenRoutes = flatAsyncRoutes(tmp.children, childrenBreadcrumb);
      childrenRoutes.map((item) => {
        res.push(item);
      });
    } else {
      const tmpBreadcrumb = [...breadcrumb];
      tmpBreadcrumb.push(tmp);
      tmp.meta.breadcrumb = tmpBreadcrumb;
      res.push(tmp);
    }
  });
  return res;
};

const views = import.meta.glob('../../views/**/**.vue');
const empty = import.meta.glob('../../layout/empty.vue');
const findRouterItemByName = (name, arr = asyncRoutes) => {
  for (let i = 0; i < arr.length; i++) {
    const item = arr[i];
    if (item.name === name) {
      return item;
    }
    if (item.children) {
      return findRouterItemByName(name, item.children);
    }
  }
  return null;
};
// 菜单转换路由
const filterAsyncRouter = (routerMap) => {
  const accessedRouters = [];
  routerMap.forEach((item) => {
    const target = findRouterItemByName(item.name);
    if (target) {
      const route = {
        path: target.path,
        name: target.name,
        // hidden: item.hidden == 1,
        meta: target.meta,
        children: item.children ? filterAsyncRouter(item.children) : null,
        component: views[`../../views${target.path}/index.vue`],
        redirect: target.redirect || null,
      };
      accessedRouters.push(route);
    }
  });
  return accessedRouters;
};

const useUserStore = defineStore('user', {
  state: () => ({
    routers: undefined,
    user: undefined,
    menus: [],
    curPageModule: '',
    headerMenus: [],
    moduleMenus: [],
  }),

  actions: {
    setToken(token) {
      local.set(import.meta.env.VITE_APP_TOKEN_PREFIX, token);
    },

    getToken() {
      return local.get(import.meta.env.VITE_APP_TOKEN_PREFIX);
    },

    clearToken() {
      local.remove(import.meta.env.VITE_APP_TOKEN_PREFIX);
    },
    setState(data) {
      this.$patch(data);
    },
    setInfo(data) {
      this.user = data;
    },

    resetUserInfo() {
      this.$reset();
    },
    setCurPageModule(name) {
      this.curPageModule = name;
    },
    setModuleMenusByName(name) {
      let item = this.menus.find((item) => item.name === name);
      this.moduleMenus = item.children;
    },
    setHeaderMenus(menus) {
      let newMenus = cloneDeep(menus);
      this.headerMenus = newMenus.map((item) => {
        delete item.children;
        return item;
      });
    },
    setMenu(data) {
      const menus = filterAsyncRouter(getTreeList(data));
      this.menus = menus;
      const routers = flatAsyncRoutes(cloneDeep(menus));
      this.routers = routers;
      routers.map((item) => router.addRoute('Layout', item));
      if (menus.length) {
        this.setHeaderMenus(menus);
        this.setCurPageModule(menus[0].name);
        this.setModuleMenusByName(menus[0].name);
      }
    },

    requestUserMenu() {
      return new Promise((resolve, reject) => {
        userApi
          .getMenus()
          .then((response) => {
            if (!response) {
              router.push({ name: 'Login' });
              return;
            }
            this.setMenu(response);
            resolve(response);
          })
          .catch((_) => {
            this.clearToken();
            router.push({ name: 'Login' });
          });
      }).catch((error) => {
        this.clearToken();
        router.push({ name: 'Login' });
      });
    },

    login(form) {
      return userApi
        .login(form)
        .then((r) => {
          this.setToken(r.token);
          return true;
        })
        .catch((e) => {
          console.error(e);
          return false;
        });
    },

    async logout() {
      await userApi.logout();
      this.clearToken();
      this.resetUserInfo();
    },
  },
});

export default useUserStore;
